<?php
/*
Plugin Name: WatuPRO Bridge for Woocommerce
Plugin URI: 
Description: This bridge allows you to sell access to your premium quizzes through Woocommerce 
Author: Kiboko Labs
Version: 1.9.1
Author URI: http://calendarscripts.info/
License: GPLv2 or later
*/

define( 'WW_BRIDGE_PATH', dirname( __FILE__ ) );
// register_activation_hook(__FILE__, 'pdf_bridge_init');

add_action('init', 'ww_bridge_init');

function ww_bridge_init() {
	global $wpdb;
	
	// catch the woocommerce action
	add_action('woocommerce_order_status_completed', 'ww_bridge_order_complete');
	add_action('template_redirect', 'ww_bridge_template_redirect');
	add_action('woocommerce_thankyou', 'ww_bridge_thankyou');
	add_action('watupro_admin_menu', 'ww_bridge_menu');
	
	// catch WatuPRO Payment page display
	add_action('watupro-display-payment-page', 'ww_bridge_display_payment_page', 10, 2);	
}

// complete Woocommerce order
function ww_bridge_order_complete($order_id) {
	global $wpdb;
		
	update_option('ww_bridge_last_order_id', $order_id);
	
	// select line items
	$items = $wpdb->get_results($wpdb->prepare("SELECT tI.*, tM.meta_value as product_id 
			FROM {$wpdb->prefix}woocommerce_order_items tI JOIN {$wpdb->prefix}woocommerce_order_itemmeta tM
			ON tM.order_item_id = tI.order_item_id AND tM.meta_key='_product_id'
			WHERE tI.order_id = %d AND tI.order_item_type = 'line_item'", $order_id));
	$quiz_ids = array(); // quiz IDs to process
	$bundle_ids = array(); // quiz bundles to process
	$certificate_ids = array(); // paid certificates to process
	$quiz_redirect = ""; // do we redirect anywhere?
	
	// now for each $item select the product, and check in the meta whether it's watupro quiz
	foreach($items as $item) {
		$product = get_post($item->product_id);
		update_option('ww_bridge_last_product_title', $product->post_title);		
		// get meta
		$atts = get_post_meta($product->ID, '_product_attributes', true);
		if(empty($atts)) $atts = [];
		
		foreach($atts as $key => $att) {		
		   // lets' always assume att values as an array 
		   $att_values = array();
		   if(!strstr($att['value'], '|')) $att_values[] = $att['value'];
		   else $att_values = explode('|', $att['value']);
			
			if($att['name'] == 'watupro' and !empty($att['value'])) {
				foreach($att_values as $att_value) $quiz_ids[] = intval($att_value);
			}
			if($att['name'] == 'watupro-bundle' and !empty($att['value'])) {
				foreach($att_values as $att_value) $bundle_ids[] = intval($att_value);
			}
			if($att['name'] == 'watupro-certificate' and !empty($att['value'])) {				
				foreach($att_values as $att_value) $certificate_ids[] = intval($att_value);
			}
			if($att['name'] == 'watupro-redirect' and empty($quiz_redirect)) $quiz_redirect = $att['value'];
		}
	}	// end foreach item	
	
	if(!empty($quiz_ids) or !empty($bundle_ids) or !empty($certificate_ids)) {
	   // select order  meta
		$user_id = get_post_meta($order_id, "_customer_user", true);
		
		if(empty($user_id)) {
			$password = wp_generate_password( 12, true );
			$user_email = get_post_meta($order_id, "_billing_email", true);
			
			// email exists?
			$user = get_user_by('email', $user_email);
			if(empty($user->ID)) {				
				$user_id = wp_create_user( $user_email, $password, $user_email );
				wp_update_user( ['ID' => $user_id, 'role' => 'student', "first_name" => get_post_meta($order_id,'_billing_first_name', true), 
					"last_name" => get_post_meta($order_id,'_billing_last_name', true) ] ) ;
			}
			else $user_id = $user->ID;
			
			// assign user to that order
			update_post_meta($order_id, '_customer_user', $user_id);
		} // end creating user
		
		// log them in
		wp_set_current_user($user_id);
      wp_set_auth_cookie($user_id);		
	}
	
	// if there are quiz ids we'll activate them but first need to ensure there is user ID
	if(!empty($quiz_ids)) {
		// now insert payments for this user ID and the given quiz IDs
		foreach($quiz_ids as $quiz_id) {
			$wpdb->query($wpdb->prepare("INSERT INTO ".WATUPRO_PAYMENTS." SET exam_id=%d, user_id=%d, date=CURDATE(),
				amount=1, status='completed', method='woocommerce'", $quiz_id, $user_id));
			
			if(is_user_logged_in() and method_exists('WatuPROPayment', 'post_process')) WatuPROPayment :: post_process($quiz_id, $user_id);	
		}
	} // end inserting quiz payments
	
	if(!empty($certificate_ids)) {
		// now insert payments for this user ID and the given quiz IDs
		foreach($certificate_ids as $certificate_id) {
			$wpdb->query($wpdb->prepare("INSERT INTO ".WATUPRO_PAYMENTS." SET certificate_id=%d, user_id=%d, date=CURDATE(),
				amount=1, status='completed', method='woocommerce'", $certificate_id, $user_id));
		}
		
		// we don't expect to have more than one certificate in the cart but just in case, we'll use the last one as a redirect
		$user_certificate = $wpdb->get_row($wpdb->prepare("SELECT ID, taking_id FROM ".WATUPRO_USER_CERTIFICATES."
			WHERE certificate_id=%d AND user_id=%d ORDER BY ID DESC LIMIT 1", $certificate_id, $user_id));
			
		if(!empty($user_certificate->ID)) $quiz_redirect = site_url("?watupro_view_certificate=1&taking_id=".$user_certificate->taking_id."&id=".$certificate_id);		
	} // end inserting certificate payment(s)
	
	if(!empty($bundle_ids)) {
	   foreach($bundle_ids as $bundle_id) {
	      $wpdb->query($wpdb->prepare("INSERT INTO ".WATUPRO_PAYMENTS." SET 
				exam_id=%d, user_id=%s, date=CURDATE(), amount=1, status='completed', 
				method='woocommerce', bundle_id=%d", 
				0, $user_id, $bundle_id));
				
			if(is_user_logged_in() and method_exists('WatuPROPayment', 'post_process_bundle')) WatuPROPayment :: post_process_bundle($bundle_id, $user_id);	
	   }
	} // end inserting bundle payments
	
	// any redirect defined?
	// if(!empty($quiz_redirect)) watupro_redirect($quiz_redirect);
	if(!empty($quiz_redirect)) update_option('watupro-woocom-redirect', $quiz_redirect);
} // end ww_bridge_order_complete

// test it by just passing order ID
// if you want to test you have to change "and false" to "and true" in the code below
function ww_bridge_template_redirect() {
	if(!empty($_GET['wwbridge_order_id']) and false) {
		ww_bridge_order_complete($_GET['wwbridge_order_id']);
		exit;
	}	
}

// this will handle redirects
function ww_bridge_thankyou($order_id)  {
   $redirect = get_option('watupro-woocom-redirect');
   if(!empty($redirect)) {
      update_option('watupro-woocom-redirect', '');
      wp_redirect($redirect);
   }
}

function ww_bridge_options() {
	global $wp, $wp_roles;
	$roles = $wp_roles->roles;
	if(!empty($_POST['ok']) and check_admin_referer('ww_bridge_options')) {
		// allow or disallow dashboard access to the selected roles
		foreach($roles as $key=>$role) {
			$r=get_role($key);
			
			if(!empty($_POST['allow_roles']) and is_array($_POST['allow_roles']) and in_array($key, $_POST['allow_roles'])) {
 				if(empty($r->capabilities['view_admin_dashboard'])) $r->add_cap('view_admin_dashboard');
			}
			else $r->remove_cap('view_admin_dashboard');
		}		
		
		update_option('watupro_ww_bridge_action', sanitize_text_field($_POST['action']));
		update_option('watupro_ww_bridge_content', watupro_strip_tags($_POST['ww_bridge_content']));
	}	
	
	$action = get_option('watupro_ww_bridge_action');
	$content = get_option('watupro_ww_bridge_content');
	$content = stripslashes($content);
	include(WW_BRIDGE_PATH . '/views/options.html.php');
}

function ww_bridge_menu() {
	add_submenu_page('watupro_exams', 'WooCommerce Bridge', 'WooCommerce Bridge', 'manage_options', 'ww_bridge__options', 'ww_bridge_options');
}

// this is called by do_action on the watupro payment page and is used to redirect to the actual WOO product page
// $exam_id is also used for certificate ID
function ww_bridge_display_payment_page($exam_id, $is_certificate = false) {
	global $wpdb, $user_ID;
	
	$action = apply_filters( 'watupro_ww_bridge_action', get_option( 'watupro_ww_bridge_action' ) );  // redirect or display text
	if(empty($action)) $action = 'REDIRECT';
	
	// is there WooCommerce product for this quiz?
	$watupro_poducts = $wpdb->get_results("SELECT tP.ID as ID, tM.meta_value as atts FROM {$wpdb->posts} tP
		JOIN {$wpdb->postmeta} tM ON tM.post_id=tP.id AND tM.meta_key = '_product_attributes'
		WHERE (tP.post_status  = 'publish' OR tP.post_status='private') AND tP.post_title!=''
		AND tM.meta_value LIKE '%watupro%'");
	$product_id = 0;
	
	foreach($watupro_poducts as $product) {
		// find if some of these atts matches the current $exam_id
		$atts = unserialize(stripslashes($product->atts));
		foreach($atts as $key => $att) {
			if(!$is_certificate and $key == 'watupro' and $att['value'] == $exam_id) {
				$product_id = $product->ID;
				break 2;
			}
			
			if($is_certificate and $key == 'watupro-certificate' and $att['value'] == $exam_id) {
				$product_id = $product->ID;
				break 2;
			}
		}
	}	
	
	if(empty($product_id)) return "";
	
	// $url = get_permalink($product_id);
	$url = apply_filters( 'watupro_ww_bridge_url', get_permalink( $product_id ), $exam_id, $is_certificate, $product_id );

	// fix odd bug with loading unfinished quiz attempts on timed quizzes
	$time_limit = $wpdb->get_var($wpdb->prepare("SELECT time_limit FROM ".WATUPRO_EXAMS." WHERE ID=%d", $exam_id));
	if($time_limit > 0 and !empty($user_ID)) {
		$wpdb->query($wpdb->prepare("DELETE FROM ".WATUPRO_TAKEN_EXAMS." WHERE in_progress=1 AND user_id=%d AND exam_id=%d", 
				$user_ID, $exam_id));
	}	

	if($action == 'REDIRECT') {
		watupro_redirect($url);
	}
	else {		
		$content = get_option('watupro_ww_bridge_content');
		$content = apply_filters( 'watupro_ww_bridge_content', $content, $exam_id, $is_certificate, $product_id );

		$content = str_replace('http://{{{url}}}', $url, stripslashes($content));
		$content = str_replace('{{{url}}}', $url, stripslashes($content));
		
		echo $content;
	}
} // end  ww_bridge_display_payment_page
